"use strict";

var _extends = function (child, parent) {
  child.prototype = Object.create(parent.prototype, {
    constructor: {
      value: child,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  child.__proto__ = parent;
};

/**
 * A simple polyfill because I can't figure out a good way to package 6to5 files
 * without this file for addons.
 */

// Some imports need require, but we manually stitch together our imports
// through concatenation.
window.require = window.require || function () {};

// Required because 6to5 tacks things onto an exports object
// which will throw an error if it doesn't exist.
/* exported exports */
var exports = {};

;(function (define) {
  define(function (require, exports, module) {
    "use strict";

    /**
     * Locals
     */

    var textContent = Object.getOwnPropertyDescriptor(Node.prototype, "textContent");
    var innerHTML = Object.getOwnPropertyDescriptor(Element.prototype, "innerHTML");
    var removeAttribute = Element.prototype.removeAttribute;
    var setAttribute = Element.prototype.setAttribute;
    var noop = function () {};

    /**
     * Register a new component.
     *
     * @param  {String} name
     * @param  {Object} props
     * @return {constructor}
     * @public
     */
    exports.register = function (name, props) {
      var baseProto = getBaseProto(props["extends"]);

      // Clean up
      delete props["extends"];

      // Pull out CSS that needs to be in the light-dom
      if (props.template) {
        var output = processCss(props.template, name);

        props.template = document.createElement("template");
        props.template.innerHTML = output.template;
        props.lightCss = output.lightCss;

        props.globalCss = props.globalCss || "";
        props.globalCss += output.globalCss;
      }

      // Inject global CSS into the document,
      // and delete as no longer needed
      injectGlobalCss(props.globalCss);
      delete props.globalCss;

      // Merge base getter/setter attributes with the user's,
      // then define the property descriptors on the prototype.
      var descriptors = Object.assign(props.attrs || {}, base.descriptors);

      // Store the orginal descriptors somewhere
      // a little more private and delete the original
      props._attrs = props.attrs;
      delete props.attrs;

      // Create the prototype, extended from base and
      // define the descriptors directly on the prototype
      var proto = createProto(baseProto, props);
      Object.defineProperties(proto, descriptors);

      // Register the custom-element and return the constructor
      return document.registerElement(name, { prototype: proto });
    };

    var base = {
      properties: {
        GaiaComponent: true,
        attributeChanged: noop,
        attached: noop,
        detached: noop,
        created: noop,

        createdCallback: function () {
          if (this.rtl) {
            addDirObserver();
          }
          injectLightCss(this);
          this.created();
        },

        /**
         * It is very common to want to keep object
         * properties in-sync with attributes,
         * for example:
         *
         *   el.value = 'foo';
         *   el.setAttribute('value', 'foo');
         *
         * So we support an object on the prototype
         * named 'attrs' to provide a consistent
         * way for component authors to define
         * these properties. When an attribute
         * changes we keep the attr[name]
         * up-to-date.
         *
         * @param  {String} name
         * @param  {String||null} from
         * @param  {String||null} to
         */
        attributeChangedCallback: function (name, from, to) {
          var prop = toCamelCase(name);
          if (this._attrs && this._attrs[prop]) {
            this[prop] = to;
          }
          this.attributeChanged(name, from, to);
        },

        attachedCallback: function () {
          this.attached();
        },
        detachedCallback: function () {
          this.detached();
        },

        /**
         * A convenient method for setting up
         * a shadow-root using the defined template.
         *
         * @return {ShadowRoot}
         */
        setupShadowRoot: function () {
          if (!this.template) {
            return;
          }
          var node = document.importNode(this.template.content, true);
          this.createShadowRoot().appendChild(node);
          return this.shadowRoot;
        },

        /**
         * Sets an attribute internally
         * and externally. This is so that
         * we can style internal shadow-dom
         * content.
         *
         * @param {String} name
         * @param {String} value
         */
        setAttr: function (name, value) {
          var internal = this.shadowRoot.firstElementChild;
          setAttribute.call(internal, name, value);
          setAttribute.call(this, name, value);
        },

        /**
         * Removes an attribute internally
         * and externally. This is so that
         * we can style internal shadow-dom
         * content.
         *
         * @param {String} name
         * @param {String} value
         */
        removeAttr: function (name) {
          var internal = this.shadowRoot.firstElementChild;
          removeAttribute.call(internal, name);
          removeAttribute.call(this, name);
        }
      },

      descriptors: {
        textContent: {
          set: function (value) {
            textContent.set.call(this, value);
            if (this.lightStyle) {
              this.appendChild(this.lightStyle);
            }
          },

          get: textContent.get
        },

        innerHTML: {
          set: function (value) {
            innerHTML.set.call(this, value);
            if (this.lightStyle) {
              this.appendChild(this.lightStyle);
            }
          },

          get: innerHTML.get
        }
      }
    };

    /**
     * The default base prototype to use
     * when `extends` is undefined.
     *
     * @type {Object}
     */
    var defaultPrototype = createProto(HTMLElement.prototype, base.properties);

    /**
     * Returns a suitable prototype based
     * on the object passed.
     *
     * @param  {HTMLElementPrototype|undefined} proto
     * @return {HTMLElementPrototype}
     * @private
     */
    function getBaseProto(proto) {
      if (!proto) {
        return defaultPrototype;
      }
      proto = proto.prototype || proto;
      return !proto.GaiaComponent ? createProto(proto, base.properties) : proto;
    }

    /**
     * Extends the given proto and mixes
     * in the given properties.
     *
     * @param  {Object} proto
     * @param  {Object} props
     * @return {Object}
     */
    function createProto(proto, props) {
      return Object.assign(Object.create(proto), props);
    }

    /**
     * Detects presence of shadow-dom
     * CSS selectors.
     *
     * @return {Boolean}
     */
    var hasShadowCSS = (function () {
      var div = document.createElement("div");
      try {
        div.querySelector(":host");return true;
      } catch (e) {
        return false;
      }
    })();

    /**
     * Regexs used to extract shadow-css
     *
     * @type {Object}
     */
    var regex = {
      shadowCss: /(?:\:host|\:\:content)[^{]*\{[^}]*\}/g,
      ":host": /(?:\:host)/g,
      ":host()": /\:host\((.+)\)(?: \:\:content)?/g,
      ":host-context": /\:host-context\((.+)\)([^{,]+)?/g,
      "::content": /(?:\:\:content)/g
    };

    /**
     * Extracts the :host and ::content rules
     * from the shadow-dom CSS and rewrites
     * them to work from the <style scoped>
     * injected at the root of the component.
     *
     * @return {String}
     */
    function processCss(template, name) {
      var globalCss = "";
      var lightCss = "";

      if (!hasShadowCSS) {
        template = template.replace(regex.shadowCss, function (match) {
          var hostContext = regex[":host-context"].exec(match);

          if (hostContext) {
            globalCss += match.replace(regex["::content"], "").replace(regex[":host-context"], "$1 " + name + "$2").replace(/ +/g, " "); // excess whitespace
          } else {
            lightCss += match.replace(regex[":host()"], name + "$1").replace(regex[":host"], name).replace(regex["::content"], name);
          }

          return "";
        });
      }

      return {
        template: template,
        lightCss: lightCss,
        globalCss: globalCss
      };
    }

    /**
     * Some CSS rules, such as @keyframes
     * and @font-face don't work inside
     * scoped or shadow <style>. So we
     * have to put them into 'global'
     * <style> in the head of the
     * document.
     *
     * @param  {String} css
     */
    function injectGlobalCss(css) {
      if (!css) return;
      var style = document.createElement("style");
      style.innerHTML = css.trim();
      headReady().then(function () {
        document.head.appendChild(style);
      });
    }


    /**
     * Resolves a promise once document.head is ready.
     *
     * @private
     */
    function headReady() {
      return new Promise(function (resolve) {
        if (document.head) {
          return resolve();
        }
        window.addEventListener("load", function fn() {
          window.removeEventListener("load", fn);
          resolve();
        });
      });
    }


    /**
     * The Gecko platform doesn't yet have
     * `::content` or `:host`, selectors,
     * without these we are unable to style
     * user-content in the light-dom from
     * within our shadow-dom style-sheet.
     *
     * To workaround this, we clone the <style>
     * node into the root of the component,
     * so our selectors are able to target
     * light-dom content.
     *
     * @private
     */
    function injectLightCss(el) {
      if (hasShadowCSS) {
        return;
      }
      el.lightStyle = document.createElement("style");
      el.lightStyle.setAttribute("scoped", "");
      el.lightStyle.innerHTML = el.lightCss;
      el.appendChild(el.lightStyle);
    }

    /**
     * Convert hyphen separated
     * string to camel-case.
     *
     * Example:
     *
     *   toCamelCase('foo-bar'); //=> 'fooBar'
     *
     * @param  {Sring} string
     * @return {String}
     */
    function toCamelCase(string) {
      return string.replace(/-(.)/g, function replacer(string, p1) {
        return p1.toUpperCase();
      });
    }

    /**
     * Observer (singleton)
     *
     * @type {MutationObserver|undefined}
     */
    var dirObserver;

    /**
     * Observes the document `dir` (direction)
     * attribute and dispatches a global event
     * when it changes.
     *
     * Components can listen to this event and
     * make internal changes if need be.
     *
     * @private
     */
    function addDirObserver() {
      if (dirObserver) {
        return;
      }

      dirObserver = new MutationObserver(onChanged);
      dirObserver.observe(document.documentElement, {
        attributeFilter: ["dir"],
        attributes: true
      });

      function onChanged(mutations) {
        document.dispatchEvent(new Event("dirchanged"));
      }
    }
  });
})(typeof define == "function" && define.amd ? define : (function (n, w) {
  "use strict";return typeof module == "object" ? function (c) {
    c(require, exports, module);
  } : function (c) {
    var m = { exports: {} };c(function (n) {
      return w[n];
    }, m.exports, m);w[n] = m.exports;
  };
})("gaia-component", this));
;(function (define) {
  define(function (require, exports, module) {
    /*jshint laxbreak:true*/
    /*jshint esnext:true*/

    "use strict";

    /**
     * Extend from the `HTMLElement` prototype
     *
     * @type {Object}
     */
    var proto = Object.create(HTMLElement.prototype);

    var innerHTML = Object.getOwnPropertyDescriptor(Element.prototype, "innerHTML");
    var removeAttribute = proto.removeAttribute;
    var setAttribute = proto.setAttribute;
    var has = Object.prototype.hasOwnProperty;

    /**
     * Runs when an instance of `GaiaTabs`
     * is first created.
     *
     * The initial value of the `select` attribute
     * is used to select a tab.
     *
     * @private
     */
    proto.createdCallback = function () {
      var _this = this;
      this.createShadowRoot().innerHTML = template;

      this.els = {
        inner: this.shadowRoot.querySelector(".dialog-inner"),
        background: this.shadowRoot.querySelector(".background"),
        window: this.shadowRoot.querySelector(".window")
      };

      this.shadowRoot.addEventListener("click", function (e) {
        return _this.onClick(e);
      });
      this.setupAnimationListeners();
      this.styleHack();
    };

    proto.onClick = function (e) {
      var el = closest("[on-click]", e.target, this);
      if (!el) {
        return;
      }
      var method = el.getAttribute("on-click");
      if (typeof this[method] == "function") {
        this[method]();
      }
    };

    proto.setupAnimationListeners = function () {
      this.addEventListener("animationstart", this.onAnimationStart.bind(this));
      this.addEventListener("animationend", this.onAnimationEnd.bind(this));
    };

    proto.styleHack = function () {
      var style = this.shadowRoot.querySelector("style").cloneNode(true);
      this.classList.add("shadow-content", "shadow-host");
      style.setAttribute("scoped", "");
      this.appendChild(style);
    };

    proto.open = function (options) {
      if (this.isOpen) {
        return;
      }
      this.animateIn(options);
      this.isOpen = true;
      this.setAttribute("opened", "");
      this.dispatch("opened");
    };

    proto.close = function (options) {
      if (!this.isOpen) {
        return;
      }
      var self = this;
      self.isOpen = false;
      this.animateOut(function () {
        self.removeAttribute("opened");
        self.dispatch("closed");
      });
    };

    proto.attributeChangedCallback = function (name, from, to) {
      if (this.attrs[name]) {
        this[name] = to;
      }
    };

    proto.setAttribute = function (attr, value) {
      this.els.inner.setAttribute(attr, value);
      setAttribute.call(this, attr, value);
    };

    proto.removeAttribute = function (attr) {
      this.els.inner.removeAttribute(attr);
      removeAttribute.call(this, attr);
    };

    proto.animateIn = function (e) {
      var hasTarget = e && ("clientX" in e || e.touches);
      if (hasTarget) {
        return this.animateInFromTarget(e);
      }

      var self = this;
      this.dispatch("animationstart");
      this.els.background.classList.add("animate-in");
      this.els.background.addEventListener("animationend", function fn() {
        self.els.background.removeEventListener("animationend", fn);
        self.els.window.classList.add("animate-in");
        self.dispatch("animationend");
      });
    };

    proto.animateInFromTarget = function (e) {
      var pos = e.touches && e.touches[0] || e;
      var scale = Math.sqrt(innerWidth * innerHeight) / 10;
      var background = this.els.background;
      var duration = scale * 7;
      var end = "transitionend";
      var self = this;

      background.style.transform = "translate(" + pos.clientX + "px, " + pos.clientY + "px)";
      background.style.transitionDuration = duration + "ms";
      background.classList.add("circular");
      this.dispatch("animationstart");

      var reflow = background.offsetTop;

      background.style.transform += " scale(" + scale + ")";
      background.style.opacity = 1;

      background.addEventListener(end, function fn() {
        background.removeEventListener(end, fn);
        self.els.window.classList.add("animate-in");
        self.dispatch("animationend");
      });
    };

    proto.animateOut = function (callback) {
      var end = "animationend";
      var background = this.els.background;
      var self = this;
      var classes = {
        el: this.classList,
        window: this.els.window.classList,
        background: this.els.background.classList
      };

      this.dispatch("animationstart");
      classes.window.add("animate-out");

      this.els.window.addEventListener(end, function fn(e) {
        self.els.window.removeEventListener(end, fn);
        e.stopPropagation();
        classes.background.add("animate-out");
        self.els.background.addEventListener(end, function fn() {
          self.els.background.removeEventListener(end, fn);
          classes.window.remove("animate-out", "animate-in");
          classes.background.remove("animate-out", "animate-in");
          background.classList.remove("circular");
          background.style = "";
          self.dispatch("animationend");
          if (callback) {
            callback();
          }
        });
      });
    };

    proto.onAnimationStart = function () {
      this.classList.add("animating");
    };

    proto.onAnimationEnd = function () {
      this.classList.remove("animating");
    };

    proto.dispatch = function (name) {
      this.dispatchEvent(new CustomEvent(name));
    };

    proto.attrs = {
      opened: {
        get: function () {
          return !!this.isOpen;
        },
        set: function (value) {
          value = value === "" || value;
          if (!value) {
            this.close();
          } else {
            this.open();
          }
        }
      }
    };

    Object.defineProperties(proto, proto.attrs);

    // Override `innerHTML` to prevent the scoped
    // stylesheet being erased by mistake.
    Object.defineProperty(proto, "innerHTML", {
      get: function () {
        return innerHTML.get.call(this);
      },
      set: function (html) {
        var style = this.querySelector("style");
        innerHTML.set.call(this, html);
        this.appendChild(style);
      }
    });

    var template = "\n<style>\n\n.shadow-content * {\n  box-sizing: border-box;\n  font-weight: inherit;\n  font-size: inherit;\n}\n\n.shadow-content p,\n.shadow-content h1,\n.shadow-content h2,\n.shadow-content h3,\n.shadow-content h4,\n.shadow-content button,\n.shadow-content fieldset {\n  padding: 0;\n  margin: 0;\n  border: 0;\n}\n\n.shadow-host {\n  display: none;\n  position: fixed;\n  top: 0px; left: 0px;\n  width: 100%;\n  height: 100%;\n  z-index: 200;\n  font-style: italic;\n  text-align: center;\n}\n\n.shadow-host[opened],\n.shadow-host.animating {\n  display: block;\n}\n\n/** Inner\n ---------------------------------------------------------*/\n\n.dialog-inner {\n  display: flex;\n  width: 100%;\n  height: 100%;\n  align-items: center;\n  justify-content: center;\n}\n\n/** Background\n ---------------------------------------------------------*/\n\n.background {\n  position: absolute;\n  top: 0; left: 0;\n  width: 100%;\n  height: 100%;\n  opacity: 0;\n  background: rgba(199,199,199,0.85);\n}\n\n/**\n * .circular\n */\n\n.background.circular {\n  width: 40px;\n  height: 40px;\n  margin: -20px;\n  border-radius: 50%;\n  will-change: transform, opacity;\n  transition-property: opacity, transform;\n  transition-timing-function: linear;\n}\n\n/**\n * .animate-in\n */\n\n.background.animate-in {\n  animation-name: gaia-dialog-fade-in;\n  animation-duration: 300ms;\n  animation-fill-mode: forwards;\n}\n\n/**\n * .animate-out\n */\n\n.background.animate-out {\n  animation-name: gaia-dialog-fade-out;\n  animation-delay: 300ms;\n  animation-duration: 300ms;\n  animation-fill-mode: forwards;\n  opacity: 1;\n}\n\n/** Window\n ---------------------------------------------------------*/\n\n.window {\n  position: relative;\n  width: 90%;\n  max-width: 350px;\n  margin: auto;\n  box-shadow: 0 1px 0 0px rgba(0,0,0,0.15);\n  background: var(--color-iota);\n  opacity: 0;\n}\n\n.window.animate-in {\n  animation-name: gaia-dialog-entrance;\n  animation-duration: 300ms;\n  animation-timing-function: cubic-bezier(0.175, 0.885, 0.320, 1.275);\n  animation-fill-mode: forwards;\n}\n\n.window.animate-out {\n  animation-name: gaia-dialog-fade-out;\n  animation-duration: 200ms;\n  animation-delay: 100ms;\n  animation-timing-function: linear;\n  animation-fill-mode: forwards;\n  opacity: 1;\n}\n\n/** Title\n ---------------------------------------------------------*/\n\n.shadow-content h1 {\n  padding: 16px;\n  font-size: 23px;\n  line-height: 26px;\n  font-weight: 200;\n  font-style: italic;\n  color: #858585;\n}\n\n.shadow-content strong {\n  font-weight: 700;\n}\n\n.shadow-content small {\n  font-size: 0.8em;\n}\n\n/** Section\n ---------------------------------------------------------*/\n\n.shadow-content section {\n  padding: 33px 18px;\n  color: #858585;\n}\n\n.shadow-content section > *:not(:last-child) {\n  margin-bottom: 13px;\n}\n\n/** Paragraphs\n ---------------------------------------------------------*/\n\n.shadow-content p {\n  text-align: left;\n}\n\n/** Buttons\n ---------------------------------------------------------*/\n\n.shadow-content button {\n  position: relative;\n  display: block;\n  width: 100%;\n  height: 50px;\n  margin: 0;\n  border: 0;\n  padding: 0rem 16px;\n  cursor: pointer;\n  font: inherit;\n  background: var(--color-beta);\n  color: var(--color-epsilon);\n  transition: all 200ms;\n  transition-delay: 300ms;\n}\n\n/**\n * .primary\n */\n\n.shadow-content button.primary {\n  color: var(--highlight-color);\n}\n\n/**\n * .danger\n */\n\n.shadow-content button.danger {\n  color: var(--color-destructive);\n}\n\n/** Button Divider Line\n ---------------------------------------------------------*/\n\n.shadow-content button:after {\n  content: '';\n  display: block;\n  position: absolute;\n  height: 1px;\n  left: 6px;\n  right: 6px;\n  top: 49px;\n  background: #E7E7E7;\n}\n\n.shadow-content button:last-of-type:after {\n  display: none;\n}\n\n.shadow-content button:active {\n  background-color: var(--highlight-color);\n  color: #fff;\n  transition: none;\n}\n\n.shadow-content button:active:after {\n  background: var(--highlight-color);\n  transition: none;\n}\n\n.shadow-content button[data-icon]:before {\n  float: left;\n}\n\n/** Fieldset (button group)\n ---------------------------------------------------------*/\n\n.shadow-content fieldset {\n  overflow: hidden;\n}\n\n.shadow-content fieldset button {\n  position: relative;\n  float: left;\n  width: 50%;\n}\n\n.shadow-content fieldset button:after {\n  content: '';\n  display: block;\n  position: absolute;\n  top: 6px;\n  bottom: 6px;\n  right: 0px;\n  left: auto;\n  width: 1px;\n  height: calc(100% - 12px);\n  background: #e7e7e7;\n  transition: all 200ms;\n  transition-delay: 200ms;\n}\n\n</style>\n\n<div class=\"dialog-inner\">\n  <div class=\"background\" on-click=\"close\"></div>\n  <div class=\"window\"><content></content></div>\n</div>";

    var animations = "\n@keyframes gaia-dialog-entrance {\n  0% {\n    opacity: 0;\n    transform: translateY(100px);\n  }\n\n  100% {\n    opacity: 1;\n    transform: translateY(0px);\n  }\n}\n\n@keyframes gaia-dialog-fade-in {\n  0% { opacity: 0 }\n  100% { opacity: 1 }\n}\n\n@keyframes gaia-dialog-fade-out {\n  0% { opacity: 1 }\n  100% { opacity: 0 }\n}";

    (function () {
      var style = document.createElement("style");
      style.innerHTML = animations;

      headReady().then(function () {
        document.head.appendChild(style);
      });

      /**
       * Resolves a promise once document.head is ready.
       *
       * @private
       */
      function headReady() {
        return new Promise(function (resolve) {
          if (document.head) {
            return resolve();
          }
          window.addEventListener("load", function fn() {
            window.removeEventListener("load", fn);
            resolve();
          });
        });
      }
    })();


    // Register and expose the constructor
    module.exports = document.registerElement("gaia-dialog", { prototype: proto });
    module.exports.proto = proto;


    module.exports.extend = function () {
      return mixin(Object.create(proto), extended);
    };

    var extended = {
      onCreated: function () {
        var self = this;

        this.createShadowRoot().innerHTML = this.template;
        this.els = { dialog: this.shadowRoot.querySelector("gaia-dialog") };
        this.setupAnimationListeners();
        this.styleHack();

        this.els.dialog.addEventListener("opened", function () {
          setAttribute.call(self, "opened", "");
        });

        this.els.dialog.addEventListener("closed", function () {
          removeAttribute.call(self, "opened");
        });
      },

      open: function (e) {
        this.els.dialog.open(e);
      },

      close: function () {
        this.els.dialog.close();
      },

      setAttribute: function (attr, value) {
        this.els.dialog.setAttribute(attr, value);
        setAttribute.call(this, attr, value);
      },

      removeAttribute: function (attr) {
        this.els.dialog.removeAttribute(attr);
        removeAttribute.call(this, attr);
      }
    };

    function closest(selector, el, top) {
      return el && el !== top ? (el.matches(selector) ? el : closest(el.parentNode)) : null;
    }

    function mixin(a, b) {
      for (var key in b) {
        a[key] = b[key];
      }
      return a;
    }
  });
})(typeof define == "function" && define.amd ? define : (function (n, w) {
  "use strict";return typeof module == "object" ? function (c) {
    c(require, exports, module);
  } : function (c) {
    var m = { exports: {} };c(function (n) {
      return w[n];
    }, m.exports, m);w[n] = m.exports;
  };
})("gaia-dialog", this));

;(function (define) {
  "use strict";define(function (require, exports, module) {
    /**
     * Dependencies
     */

    var component = require("gaia-component");

    /**
     * Simple logger (toggle 0)
     *
     * @type {Function}
     */
    var debug = 0 ? console.log.bind(console) : function () {};

    var NODE_TYPES = {
      1: "element",
      3: "text"
    };

    var OBSERVER_CONFIG = {
      childList: true,
      attributes: true,
      characterData: true,
      subtree: true
    };

    /**
     * Register the element.
     *
     * @return {Element} constructor
     */
    module.exports = component.register("gaia-dom-tree", {
      /**
       * Called when the element is first created.
       *
       * Here we create the shadow-root and
       * inject our template into it.
       *
       * @private
       */
      created: function () {
        var _this2 = this;
        debug("created");
        this.setupShadowRoot();
        this.els = {
          inner: this.shadowRoot.querySelector(".inner"),
          tree: this.shadowRoot.querySelector(".tree")
        };

        this.filter = this.getAttribute("filter");

        this.selectedTreeNode = null;
        this.selectedNode = null;
        this.treeMap = new Map();
        this.els.inner.addEventListener("click", function (e) {
          return _this2.onInnerClick(e);
        });
        this.els.inner.addEventListener("contextmenu", function (e) {
          return _this2.onInnerClick(e);
        });
      },

      attrs: {
        filter: {
          get: function () {
            return this._filter;
          },
          set: function (value) {
            if (value === this._filter) {
              return;
            }
            this._filter = value;
            this.setAttr("filter", value);
            this.render();
          }
        }
      },

      setRoot: function (el) {
        debug("set root", el);
        this.root = el;
      },

      render: function () {
        if (!this.root) {
          return;
        }

        var tree = this.createTree(this.root);
        this.els.tree.innerHTML = "";
        this.els.tree.appendChild(tree);
        debug("rendered");
        this.setMutationObserver();
      },

      watchChanges: function () {
        this.observer.observe(this.root, OBSERVER_CONFIG);
      },

      unwatchChanges: function () {
        this.observer.disconnect();
      },

      setMutationObserver: function () {
        var _this3 = this;
        if (this.observer) {
          return;
        }

        this.observer = new MutationObserver(function (mutations) {
          // Only re-render if a mutation occurred outside of
          // the <gaia-dom-tree> shadow root.
          for (var i = mutations.length - 1; i >= 0; i--) {
            if (!_this3.shadowRoot.contains(mutations[i].target)) {
              _this3.render();
              return;
            }
          }
        });

        this.watchChanges();
      },

      select: function (el) {
        debug("select", el);
        var treeNode = this.treeMap.get(el);
        if (!treeNode) return;
        this.selectNode(treeNode);
        this.expandNode(treeNode);
      },

      onInnerClick: function (e) {
        debug("inner click", e);
        if (e.type === "contextmenu") e.preventDefault();
        var nodeTitle = e.target.closest("h3");
        if (nodeTitle) this.onNodeTitleClick(nodeTitle, e);
      },

      onNodeTitleClick: function (el, e) {
        var node = el.closest("li");
        this.toggleExpanded(node);
        this.selectNode(node, e);
      },

      expandNode: function (node) {
        debug("expand node", node);
        node.classList.add("expanded");
        var parent = node.parentNode.closest("li");
        if (parent) this.expandNode(parent);
      },

      contractNode: function (node) {
        node.classList.remove("expanded");
      },

      toggleExpanded: function (node) {
        var expanded = node.classList.contains("expanded");
        if (expanded) this.contractNode(node);else this.expandNode(node);
      },

      selectNode: function (treeNode, e) {
        var previous = this.selectedTreeNode;
        if (previous) previous.classList.remove("selected");
        treeNode.classList.add("selected");
        this.selectedTreeNode = treeNode;
        this.selectedNode = treeNode.sourceNode;
        this.dispatch((e && e.type === "contextmenu") ? "longpressed" : "click");
      },

      dispatch: function (name) {
        this.dispatchEvent(new Event(name));
      },

      createTree: function (node) {
        var _this4 = this;
        var treeNode = document.createElement("li");
        var title = document.createElement("h3");
        var children = document.createElement("ul");
        var type = NODE_TYPES[node.nodeType];
        var stringified = stringify[type](node);

        // Exclude <gaia-dom-tree> from being seen as
        // part of the DOM tree.
        var childNodes = [].filter.call(node.childNodes, function (node) {
          return node.nodeType === Node.TEXT_NODE || (node.nodeType === Node.ELEMENT_NODE && node.nodeName !== "GAIA-DOM-TREE" && ((_this4.filter && !node.matches(_this4.filter)) || !_this4.filter));
        });

        if (!stringified) return;

        title.innerHTML = stringified;

        treeNode.appendChild(title);
        treeNode.appendChild(children);
        treeNode.sourceNode = node;
        treeNode.classList.add(type);

        // Flag when a node has children
        treeNode.classList.toggle("has-children", !!childNodes.length);

        childNodes.map(function (node) {
          return _this4.createTree(node);
        }).filter(function (node) {
          return !!node;
        }).forEach(children.appendChild, children);

        this.treeMap.set(node, treeNode);
        return treeNode;
      },

      template: "\n    <style>\n\n      * { box-sizing: border-box; }\n\n      :host {\n        display: block;\n      }\n\n      .inner {\n        color: var(--text-color);\n        background: var(--background);\n        font-family: Consolas,Monaco,\"Andale Mono\",monospace;\n        line-height: 1;\n        -moz-user-select: none;\n        cursor: default;\n      }\n\n      .tree {\n        overflow: auto;\n      }\n\n      ul {\n        list-style-type: none;\n        padding: 0;\n        margin: 0;\n      }\n\n      /** Node Title\n       ---------------------------------------------------------*/\n\n      h3 {\n        display: inline-block;\n        width: 100%;\n        margin: 0;\n        padding: 0.2em 0.2em;\n        font: inherit;\n        white-space: nowrap;\n      }\n\n      /**\n       * .selected\n       */\n\n      li.selected > h3 {\n        background: var(--background-plus)\n      }\n\n      li.text > h3 {\n        font-style: italic;\n        opacity: 0.4;\n      }\n\n      /** Node Icon\n       ---------------------------------------------------------*/\n\n      li > h3:before {\n        content: '';\n        display: inline-block;\n        vertical-align: to;\n        text-align: center;\n        width: 1.4em;\n        font-size: 0.8em;\n        opacity: 0.2\n      }\n\n      li.element.has-children > h3:before {\n        content: '\u25b6';\n      }\n\n      /**\n       * .expanded\n       */\n\n      li.expanded > h3:before {\n        transform: rotate(90deg);\n      }\n\n      /** Node Attributes\n       ---------------------------------------------------------*/\n\n      .attr-value {\n        color: var(--highlight-color);\n      }\n\n      /** Children\n       ---------------------------------------------------------*/\n\n      li > ul {\n        display: none;\n        -moz-padding-start: 0.8em;\n      }\n\n      /**\n       * .expanded\n       */\n\n      li.expanded > ul {\n        display: block;\n      }\n\n    </style>\n    <div class=\"inner\">\n      <ul class=\"tree\"></div>\n    </div>\n  "
    });

    var stringify = {
      element: function (el) {
        var tagName = el.tagName.toLowerCase();
        var attrs = stringifyAttrs(el.attributes);
        return "&lt;" + tagName + "<span class=\"attrs\">" + attrs + "</span>&gt;";
      },

      text: function (textNode) {
        var value = textNode.nodeValue.trim();
        debug("stringify textNode", value);
        return value;
      }
    };

    function stringifyAttrs(attrs) {
      return [].map.call(attrs, function (attr) {
        return " " + attr.name + "=&quot;<span class=\"attr-value\">" + attr.value + "</span>&quot;";
      }).join("");
    }

    function escape(html) {
      return html.replace(/&/g, "&amp;").replace(/</g, "&lt;").replace(/>/g, "&gt;").replace(/"/g, "&quot;").replace(/'/g, "&#039;");
    }
  });
})(typeof define == "function" && define.amd ? define : (function (n, w) {
  "use strict";return typeof module == "object" ? function (c) {
    c(require, exports, module);
  } : function (c) {
    var m = { exports: {} };c(function (n) {
      return w[n];
    }, m.exports, m);w[n] = m.exports;
  };
})("gaia-dom-tree", this));

;(function (define) {
  "use strict";define(function (require, exports, module) {
    /**
     * Dependencies
     */

    var component = require("gaia-component");

    /**
     * Simple logger (toggle 0)
     *
     * @type {Function}
     */
    var debug = 1 ? console.log.bind(console) : function () {};

    /**
     * Register the element.
     *
     * @return {Element} constructor
     */
    module.exports = component.register("gaia-css-inspector", {
      /**
       * Called when the element is first created.
       *
       * Here we create the shadow-root and
       * inject our template into it.
       *
       * @private
       */
      created: function () {
        debug("created");
        this.setupShadowRoot();
        this.els = {
          inner: this.shadowRoot.querySelector(".inner"),
          styles: this.shadowRoot.querySelector(".styles")
        };
      },

      set: function (el) {
        debug("set element", el);
        this.root = el;
        this.render();
      },

      render: function () {
        var styles = getStylesForElement(this.root);
        this.renderStyles(styles);
      },

      renderStyles: function (styles) {
        this.els.styles.innerHTML = "";
        for (var prop in styles) {
          var li = document.createElement("li");
          li.innerHTML = "<span class=\"prop\">" + prop + ": </span><span class=\"value\">" + styles[prop] + "</span>";
          this.els.styles.appendChild(li);
        }
      },

      template: "\n    <style>\n      * { box-sizing: border-box; }\n\n      :host {\n        display: block;\n      }\n\n      .inner {\n        color: var(--text-color);\n        background: var(--background);\n        font-family: Consolas,Monaco,\"Andale Mono\",monospace;\n        line-height: 1;\n        -moz-user-select: none;\n        cursor: default;\n      }\n\n      .styles {\n        list-style-type: none;\n        margin: 0;\n        padding: 0;\n      }\n\n      .styles li {\n        margin-bottom: 0.27em;\n      }\n\n\n      .styles li > .prop {\n\n      }\n\n      .styles li > .value {\n        color: var(--highlight-color);\n      }\n\n    </style>\n    <div class=\"inner\">\n      <ul class=\"styles\"></ul>\n    </div>\n  "
    });

    function getStylesForElement(el) {
      var styles = {};

      [].forEach.call(document.styleSheets, function (sheet) {
        [].forEach.call(sheet.cssRules, function (rule) {
          if (!el.matches(rule.selectorText)) return;
          debug("matching rule", rule);
          var style = rule.style;
          [].forEach.call(style, function (prop) {
            styles[prop] = style[prop];
          });
        });
      });

      debug("got styles", styles);
      return styles;
    }
  });
})(typeof define == "function" && define.amd ? define : (function (n, w) {
  "use strict";return typeof module == "object" ? function (c) {
    c(require, exports, module);
  } : function (c) {
    var m = { exports: {} };c(function (n) {
      return w[n];
    }, m.exports, m);w[n] = m.exports;
  };
})("gaia-css-inspector", this));

;(function (define) {
  "use strict";define(function (require, exports, module) {
    /**
     * Dependencies
     */

    var component = require("gaia-component");

    /**
     * Register the element.
     *
     * @return {Element} constructor
     */
    module.exports = component.register("gaia-modal", {
      /**
       * Called when the element is first created.
       *
       * Here we create the shadow-root and
       * inject our template into it.
       *
       * @private
       */
      created: function () {
        this.setupShadowRoot();

        // Properties
        this.active = this.getAttribute("active");
      },

      /**
       * Known attribute property
       * descriptors.
       *
       * These setters get called when matching
       * attributes change on the element.
       *
       * @type {Object}
       */
      attrs: {
        active: {
          get: function () {
            return this._active || false;
          },
          set: function (value) {
            value = !!(value || value === "");

            if (value === this.active) {
              return;
            }
            this._active = value;

            if (value) {
              this.setAttr("active", "");
            } else {
              this.removeAttr("active");
            }
          }
        }
      },

      open: function () {
        this.active = true;
      },

      close: function () {
        this.active = false;
      },

      template: "<div class=\"inner\">\n    <content></content>\n  </div>\n\n  <style>\n  :host {\n    position: fixed;\n    top: 0;\n    left: 0;\n    width: 100%;\n    height: 100%;\n    z-index: 9999;\n    transition: all 0.2s;\n    transform: translate(0, 100%);\n    opacity: 0;\n    pointer-events: none;\n  }\n\n  :host[active] {\n    transform: translate(0, 0);\n    opacity: 1;\n    pointer-events: auto;\n  }\n\n  .inner {\n    background: var(--background, #fff);\n    position: absolute;\n    top: 0;\n    left: 0;\n    width: 100%;\n    height: 100%;\n  }\n  </style>"
    });
  });
})(typeof define == "function" && define.amd ? define : (function (n, w) {
  "use strict";return typeof module == "object" ? function (c) {
    c(require, exports, module);
  } : function (c) {
    var m = { exports: {} };c(function (n) {
      return w[n];
    }, m.exports, m);w[n] = m.exports;
  };
})("gaia-modal", this));

var Model = (function () {
  var Model = function Model(properties) {
    var _this5 = this;
    properties = properties || {};

    for (var key in properties) {
      this[key] = properties[key];
    }

    this.observableProperties = {};

    Object.observe(this, function (changes) {
      changes.forEach(function (change) {
        var handlers = _this5.observableProperties[change.name];
        if (handlers) {
          handlers.forEach(function (handler) {
            handler(change);
          });
        }
      });
    });
  };

  Model.prototype.on = function (property, handler) {
    if (typeof handler !== "function") {
      return;
    }

    if (!this.observableProperties[property]) {
      ObserveUtils.defineObservableProperties(this, property);
      this.observableProperties[property] = [];
    }

    this.observableProperties[property].push(handler);
  };

  return Model;
})();

exports.Model = Model;


/**
 * View
 */
var events = {};

var View = (function () {
  var View = function View(options) {
    options = options || {};

    for (var key in options) {
      this[key] = options[key];
    }

    if (!this.el) {
      this.el = document.createElement("div");
    }
  };

  View.prototype.init = function (controller) {
    this.controller = controller;

    return this;
  };

  View.prototype.layout = function (template) {
    return template;
  };

  View.prototype.render = function (params) {
    var innerHTML = "";

    if (params) {
      if (typeof params === "object") {
        for (var index in params) {
          var param = params[index];
          innerHTML += this.template(param);
        }
      } else {
        for (var i = 0; i < params.length; i++) {
          var param = params[i];
          innerHTML += this.template(param);
        }
      }
    } else {
      innerHTML = this.template();
    }

    this.el.innerHTML = this.layout(innerHTML);
  };

  View.prototype.template = function () {
    return "";
  };

  View.prototype.$ = function (selector) {
    return this.el.querySelector(selector);
  };

  View.prototype.$$ = function (selector) {
    return this.el.querySelectorAll(selector);
  };

  View.prototype.on = function (type, selector, handler) {
    if (!events[type]) {
      events[type] = [];
      window.addEventListener(type, delegateHandler, true);
    }

    events[type].push({
      selector: selector,
      handler: handler
    });
  };

  View.prototype.off = function (type, selector, handler) {
    if (!events[type]) {
      return;
    }

    events[type] = events[type].filter(function (delegate) {
      if (typeof handler === "function") {
        return delegate.selector !== selector || delegate.handler !== handler;
      }

      return delegate.selector !== selector;
    });
  };

  return View;
})();

exports.View = View;


function delegateHandler(event) {
  var target = event.target;

  events[event.type].forEach(function (delegate) {
    if (target.matches(delegate.selector)) {
      delegate.handler.call(target, event);
    }
  });
}

var Controller = (function () {
  var Controller = function Controller(options) {
    options = options || {};

    for (var key in options) {
      this[key] = options[key];
    }

    // Initialize the view (if applicable) when the
    // controller is instantiated.
    if (this.view && typeof this.view.init === "function") {
      this.view.init(this);
    }
  };

  Controller.prototype.teardown = function () {};

  Controller.prototype.main = function () {};

  return Controller;
})();

exports.Controller = Controller;
var RoutingController = (function (Controller) {
  var RoutingController = function RoutingController(controllers) {
    Controller.call(this);
    this.controllers = controllers;
    this.activeController = null;
    window.addEventListener("hashchange", this.route.bind(this));
  };

  _extends(RoutingController, Controller);

  RoutingController.prototype.route = function () {
    var route = window.location.hash.slice(1);
    var controller = this.controllers[route];
    if (controller) {
      if (this.activeController) {
        this.activeController.teardown();
      }

      this.activeController = controller;
      controller.main();
    }
  };

  return RoutingController;
})(Controller);

exports.RoutingController = RoutingController;
var Service = (function () {
  var Service = function Service() {
    this._listeners = {};
    this._dispatchedEvents = {};
  };

  Service.prototype.addEventListener = function (name, callback, trigger) {
    var _this6 = this;
    if (!this._listeners[name]) {
      this._listeners[name] = [callback];
    } else {
      this._listeners[name].push(callback);
    }

    if (trigger && this._dispatchedEvents[name] !== undefined) {
      setTimeout(function () {
        callback(_this6._dispatchedEvents[name]);
      });
    }
  };

  Service.prototype.removeEventListener = function (name, callback) {
    if (!this._listeners[name]) {
      return;
    }

    var listenerIndex;
    this._listeners[name].find(function (listener, index) {
      if (listener === callback) {
        listenerIndex = index;
      }

      return listenerIndex !== undefined;
    });

    if (listenerIndex !== undefined) {
      this._listeners[name].splice(listenerIndex, 1);
    }
  };

  Service.prototype._dispatchEvent = function (name, params) {
    if (!this._listeners[name]) {
      return;
    }

    this._dispatchedEvents[name] = params || null;

    this._listeners[name].forEach(function (listener) {
      listener(params);
    });
  };

  return Service;
})();

exports.Service = Service;


(function (window) {
  "use strict";

  var proto = Object.create(HTMLElement.prototype);

  var lightHTML = "<style>\nfxos-customizer-highlighter {\n  pointer-events: none;\n  display: none;\n  position: fixed;\n  z-index: 9999999;\n}\n</style>";

  var shadowHTML = "<style scoped>\n.inner {\n  background-color: #00caf2;\n  border: 2px dotted #fff;\n  outline: 1px solid #00caf2;\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  opacity: 0.75;\n}\n</style>\n<div class=\"inner\"></div>";

  proto.createdCallback = function () {
    var shadow = this.createShadowRoot();
    shadow.innerHTML = shadowHTML;

    this.innerHTML = lightHTML;
  };

  proto.show = function () {
    this.style.display = "block";
  };

  proto.hide = function () {
    this.style.display = "none";
  };

  proto.clearTarget = function () {
    this.setTargetRect(0, 0, 0, 0);

    this.hide();
  };

  proto.setTargetRect = function (x, y, w, h) {
    this.style.left = x + "px";
    this.style.top = y + "px";
    this.style.width = w + "px";
    this.style.height = h + "px";

    this.show();
  };

  proto.setTargetElement = function (element) {
    var rect = element.getBoundingClientRect();
    this.setTargetRect(rect.left, rect.top, rect.width, rect.height);
  };

  var FXOSCustomizerHighlighter = document.registerElement("fxos-customizer-highlighter", {
    prototype: proto
  });

  window.FXOSCustomizerHighlighter = FXOSCustomizerHighlighter;
})(window);

(function (window) {
  "use strict";

  var proto = Object.create(HTMLElement.prototype);

  var template = "<gaia-dom-tree></gaia-dom-tree>\n<gaia-css-inspector></gaia-css-inspector>\n<gaia-modal>\n  <p>lorem ipsum...</p>\n</gaia-modal>";

  proto.createdCallback = function () {
    var shadow = this.createShadowRoot();
    shadow.innerHTML = template;

    this.gaiaDomTree = shadow.querySelector("gaia-dom-tree");
    this.gaiaCssInspector = shadow.querySelector("gaia-css-inspector");
    this.gaiaModal = shadow.querySelector("gaia-modal");

    this.gaiaDomTree.addEventListener("selected", this._handleSelected.bind(this));
    this.gaiaDomTree.addEventListener("longpressed", this._handleLongPressed.bind(this));
  };

  proto.setRootNode = function (rootNode) {
    rootNode.addEventListener("click", this._handleClick.bind(this));

    this.gaiaDomTree.setRoot(rootNode);
    this.gaiaDomTree.render();
  };

  proto._handleSelected = function (e) {
    var prev = this._selected;
    this._selected = this.gaiaDomTree.selectedNode;

    if (prev) {
      prev.classList.remove("selected");
    }

    if (this._selected.nodeType == 3) {
      this._selected = this._selected.parentNode;
    }
  };

  proto._handleLongPressed = function (e) {
    this._handleSelected(e);

    this.dispatchEvent(new CustomEvent("action", {
      detail: this._selected
    }));
  };

  proto._handleClick = function (e) {
    if (e.target === this.gaiaDomTree) {
      return;
    }

    this.gaiaDomTree.select(e.target);
  };

  document.registerElement("fxos-customizer", {
    prototype: proto
  });
})(window);

var ActionMenuController = (function (Controller) {
  var ActionMenuController = function ActionMenuController(options) {
    Controller.call(this, options);
  };

  _extends(ActionMenuController, Controller);

  ActionMenuController.prototype.open = function (target) {
    this.target = target;

    this.view.dialog.open();
  };

  ActionMenuController.prototype.close = function () {
    this.view.dialog.close();
  };

  ActionMenuController.prototype.edit = function () {
    this.editController.open(this.target);
  };

  ActionMenuController.prototype.remove = function () {
    this.target.remove();
  };

  ActionMenuController.prototype.move = function () {};

  return ActionMenuController;
})(Controller);

exports["default"] = ActionMenuController;
var EditController = (function (Controller) {
  var EditController = function EditController(options) {
    Controller.call(this, options);
  };

  _extends(EditController, Controller);

  EditController.prototype.open = function (target) {
    this.target = target;

    this.view.modal.open();
  };

  EditController.prototype.close = function () {
    this.view.modal.close();
  };

  return EditController;
})(Controller);

exports["default"] = EditController;
var MainController = (function (Controller) {
  var MainController = function MainController(options) {
    Controller.call(this, options);

    window.addEventListener("contextmenu", this.onContextMenu.bind(this));
  };

  _extends(MainController, Controller);

  MainController.prototype.attachView = function () {
    document.body.appendChild(this.view.el);
  };

  MainController.prototype.removeView = function () {
    document.body.removeChild(this.view.el);
  };

  MainController.prototype.onContextMenu = function () {
    this.view.customizer.setRootNode(document.getElementById("root"));
    this.attachView();
  };

  return MainController;
})(Controller);

exports["default"] = MainController;


/* global View */

var actionMenuViewTemplate = "<gaia-dialog>\n  <button type=\"button\" data-action=\"edit\">Edit</button>\n  <button type=\"button\" data-action=\"remove\">Remove</button>\n  <button type=\"button\" data-action=\"move\">Move</button>\n  <button type=\"button\" data-action=\"cancel\">Cancel</button>\n</gaia-dialog>";

var ActionMenuView = (function (View) {
  var ActionMenuView = function ActionMenuView(options) {
    View.call(this, options);

    this.render();
  };

  _extends(ActionMenuView, View);

  ActionMenuView.prototype.init = function (controller) {
    var _this7 = this;
    View.prototype.init.call(this, controller);

    this.dialog = this.$("gaia-dialog");

    this.on("click", "button", function (evt) {
      var action = _this7.controller[evt.target.dataset.action];
      if (typeof action === "function") {
        action.call(_this7.controller);
      }

      _this7.dialog.close();
    });
  };

  ActionMenuView.prototype.template = function () {
    return actionMenuViewTemplate;
  };

  return ActionMenuView;
})(View);

exports["default"] = ActionMenuView;


/* global View */

var editViewTemplate = "<gaia-modal>Editor goes here</gaia-modal>";

var EditView = (function (View) {
  var EditView = function EditView(options) {
    View.call(this, options);

    this.render();
  };

  _extends(EditView, View);

  EditView.prototype.init = function (controller) {
    View.prototype.init.call(this, controller);

    this.modal = this.$("gaia-modal");
  };

  EditView.prototype.template = function () {
    return editViewTemplate;
  };

  return EditView;
})(View);

exports["default"] = EditView;


/* global View */

var mainViewTemplate = "<fxos-customizer></fxos-customizer>\n<fxos-customizer-highlighter></fxos-customizer-highlighter>";

var MainView = (function (View) {
  var MainView = function MainView(options) {
    View.call(this, options);

    this.render();
  };

  _extends(MainView, View);

  MainView.prototype.init = function (controller) {
    var _this8 = this;
    View.prototype.init.call(this, controller);

    this.customizer = this.$("fxos-customizer");
    this.highlighter = this.$("fxos-customizer-highlighter");

    this.on("action", "fxos-customizer", function (evt) {
      _this8.controller.actionMenuController.open(evt.detail);
    });
  };

  MainView.prototype.template = function () {
    return mainViewTemplate;
  };

  MainView.prototype.render = function () {
    View.prototype.render.call(this);

    this.el.appendChild(this.actionMenuView.el);
    this.el.appendChild(this.editView.el);
  };

  return MainView;
})(View);

exports["default"] = MainView;


/*global EditView*/
/*global ActionMenuView*/
/*global MainView*/

/*global EditController*/
/*global ActionMenuController*/
/*global MainController*/

var editView = new EditView();
var actionMenuView = new ActionMenuView();
var mainView = new MainView({
  actionMenuView: actionMenuView,
  editView: editView
});

var editController = new EditController({
  view: editView
});

var actionMenuController = new ActionMenuController({
  view: actionMenuView,

  editController: editController
});

window.mainController = new MainController({
  view: mainView,

  actionMenuController: actionMenuController
});