define(["exports"], function (exports) {
  "use strict";

  (function (window) {
    "use strict";

    var proto = Object.create(HTMLElement.prototype);

    var lightHTML = "<style>\nfxos-customizer-highlighter {\n  pointer-events: none;\n  display: none;\n  position: fixed;\n  z-index: 9999999;\n}\n</style>";

    var shadowHTML = "<style scoped>\n.inner {\n  background-color: #00caf2;\n  border: 2px dotted #fff;\n  outline: 1px solid #00caf2;\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 100%;\n  height: 100%;\n  opacity: 0.75;\n}\n</style>\n<div class=\"inner\"></div>";

    proto.createdCallback = function () {
      var shadow = this.createShadowRoot();
      shadow.innerHTML = shadowHTML;

      this.innerHTML = lightHTML;
    };

    proto.show = function () {
      this.style.display = "block";
    };

    proto.hide = function () {
      this.style.display = "none";
    };

    proto.clearTarget = function () {
      this.setTargetRect(0, 0, 0, 0);

      this.hide();
    };

    proto.setTargetRect = function (x, y, w, h) {
      this.style.left = x + "px";
      this.style.top = y + "px";
      this.style.width = w + "px";
      this.style.height = h + "px";

      this.show();
    };

    proto.setTargetElement = function (element) {
      var rect = element.getBoundingClientRect();
      this.setTargetRect(rect.left, rect.top, rect.width, rect.height);
    };

    var FXOSCustomizerHighlighter = document.registerElement("fxos-customizer-highlighter", {
      prototype: proto
    });

    window.FXOSCustomizerHighlighter = FXOSCustomizerHighlighter;
  })(window);
});